/* Sample application to demonstrat FT800 primitives, widgets and customized screen shots */

#include "FT_Platform.h"


#define SAMAPP_DELAY_BTW_APIS (1000)
#define SAMAPP_ENABLE_DELAY() Ft_Gpu_Hal_Sleep(SAMAPP_DELAY_BTW_APIS)
#define SAMAPP_ENABLE_DELAY_VALUE(x) Ft_Gpu_Hal_Sleep(x)
#define F16(s)        ((ft_int32_t)((s) * 65536))
#define SQ(v) (v*v)
#define MEMCMD(a) Ft_Gpu_Hal_WrCmdBuf(phost,a,sizeof(a))




#define imw  (80)
#define imh  (53)

/* Global variables for display resolution to support various display panels */
/* Default is WQVGA - 480x272 */
ft_int16_t FT_DispWidth = 480;
ft_int16_t FT_DispHeight = 272;
ft_int16_t FT_DispHCycle =  548;
ft_int16_t FT_DispHOffset = 43;
ft_int16_t FT_DispHSync0 = 0;
ft_int16_t FT_DispHSync1 = 41;
ft_int16_t FT_DispVCycle = 292;
ft_int16_t FT_DispVOffset = 12;
ft_int16_t FT_DispVSync0 = 0;
ft_int16_t FT_DispVSync1 = 10;
ft_uint8_t FT_DispPCLK = 5;
ft_char8_t FT_DispSwizzle = 0;
ft_char8_t FT_DispPCLKPol = 1;

/* Global used for buffer optimization */
Ft_Gpu_Hal_Context_t host,*phost;



ft_int16_t CartX = 480;

ft_uint32_t Ft_CmdBuffer_Index;
ft_uint32_t Ft_DlBuffer_Index;

#ifdef BUFFER_OPTIMIZATION
ft_uint8_t  Ft_DlBuffer[FT_DL_SIZE];
ft_uint8_t  Ft_CmdBuffer[FT_CMD_FIFO_SIZE];
#endif

ft_void_t Ft_App_WrCoCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef  BUFFER_OPTIMIZATION
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_CmdBuffer[Ft_CmdBuffer_Index];
   *pBuffcmd = cmd;
#endif
#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_WrCmd32(phost,cmd);
#endif

   /* Increment the command index */
   Ft_CmdBuffer_Index += FT_CMD_SIZE;  
}

ft_void_t Ft_App_WrDlCmd_Buffer(Ft_Gpu_Hal_Context_t *phost,ft_uint32_t cmd)
{
#ifdef BUFFER_OPTIMIZATION  
   /* Copy the command instruction into buffer */
   ft_uint32_t *pBuffcmd;
   pBuffcmd =(ft_uint32_t*)&Ft_DlBuffer[Ft_DlBuffer_Index];
   *pBuffcmd = cmd;
#endif

#ifdef ARDUINO_PLATFORM
   Ft_Gpu_Hal_Wr32(phost,(RAM_DL+Ft_DlBuffer_Index),cmd);
#endif
   /* Increment the command index */
   Ft_DlBuffer_Index += FT_CMD_SIZE;  
}

ft_void_t Ft_App_WrCoStr_Buffer(Ft_Gpu_Hal_Context_t *phost,const ft_char8_t *s)
{
#ifdef  BUFFER_OPTIMIZATION  
  ft_uint16_t length = 0;
  length = strlen(s) + 1;//last for the null termination
  
  strcpy(&Ft_CmdBuffer[Ft_CmdBuffer_Index],s);  

  /* increment the length and align it by 4 bytes */
  Ft_CmdBuffer_Index += ((length + 3) & ~3);  
#endif  
}

ft_void_t Ft_App_Flush_DL_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_DlBuffer_Index> 0)
     Ft_Gpu_Hal_WrMem(phost,RAM_DL,Ft_DlBuffer,Ft_DlBuffer_Index);
#endif     
   Ft_DlBuffer_Index = 0;
   
}

ft_void_t Ft_App_Flush_Co_Buffer(Ft_Gpu_Hal_Context_t *phost)
{
#ifdef  BUFFER_OPTIMIZATION    
   if (Ft_CmdBuffer_Index > 0)
     Ft_Gpu_Hal_WrCmdBuf(phost,Ft_CmdBuffer,Ft_CmdBuffer_Index);
#endif     
   Ft_CmdBuffer_Index = 0;
}


/* API to give fadeout effect by changing the display PWM from 100 till 0 */
ft_void_t SAMAPP_fadeout()
{
   ft_int32_t i;
	
	for (i = 100; i >= 0; i -= 3) 
	{
		Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);

		Ft_Gpu_Hal_Sleep(2);//sleep for 2 ms
	}
}

/* API to perform display fadein effect by changing the display PWM from 0 till 100 and finally 128 */
ft_void_t SAMAPP_fadein()
{
	ft_int32_t i;
	
	for (i = 0; i <=100 ; i += 3) 
	{
		Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);
		Ft_Gpu_Hal_Sleep(2);//sleep for 2 ms
	}
	/* Finally make the PWM 100% */
	i = 128;
	Ft_Gpu_Hal_Wr8(phost,REG_PWM_DUTY,i);
}


/* API to check the status of previous DLSWAP and perform DLSWAP of new DL */
/* Check for the status of previous DLSWAP and if still not done wait for few ms and check again */
ft_void_t SAMAPP_GPU_DLSwap(ft_uint8_t DL_Swap_Type)
{
	ft_uint8_t Swap_Type = DLSWAP_FRAME,Swap_Done = DLSWAP_FRAME;

	if(DL_Swap_Type == DLSWAP_LINE)
	{
		Swap_Type = DLSWAP_LINE;
	}

	/* Perform a new DL swap */
	Ft_Gpu_Hal_Wr8(phost,REG_DLSWAP,Swap_Type);

	/* Wait till the swap is done */
	while(Swap_Done)
	{
		Swap_Done = Ft_Gpu_Hal_Rd8(phost,REG_DLSWAP);

		if(DLSWAP_DONE != Swap_Done)
		{
			Ft_Gpu_Hal_Sleep(10);//wait for 10ms
		}
	}	
}


ft_void_t Ft_BootupConfig()
{

	/* Do a power cycle for safer side */
	Ft_Gpu_Hal_Powercycle(phost,FT_TRUE);
	 Ft_Gpu_Hal_Rd16(phost,RAM_G);
	/* Set the clk to external clock */
	Ft_Gpu_HostCommand(phost,FT_GPU_EXTERNAL_OSC);  
	Ft_Gpu_Hal_Sleep(10);
	  

	/* Switch PLL output to 48MHz */
	Ft_Gpu_HostCommand(phost,FT_GPU_PLL_48M);  
	Ft_Gpu_Hal_Sleep(10);

	/* Do a core reset for safer side */
	Ft_Gpu_HostCommand(phost,FT_GPU_CORE_RESET);     

	/* Access address 0 to wake up the FT800 */
	Ft_Gpu_HostCommand(phost,FT_GPU_ACTIVE_M);  

    Ft_Gpu_Hal_Wr8(phost, REG_GPIO_DIR,0x80 | Ft_Gpu_Hal_Rd8(phost,REG_GPIO_DIR));
    Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0x080 | Ft_Gpu_Hal_Rd8(phost,REG_GPIO));
	
	{
		ft_uint8_t chipid;
		//Read Register ID to check if FT800 is ready. 
		chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
		while(chipid != 0x7C)
			chipid = Ft_Gpu_Hal_Rd8(phost, REG_ID);
#ifdef MSVC_PLATFORM
		printf("VC1 register ID after wake up %x\n",chipid);
#endif
	}
	/* Configuration of LCD display */
#ifdef SAMAPP_DISPLAY_QVGA	
	/* Values specific to QVGA LCD display */
	FT_DispWidth = 320;
	FT_DispHeight = 240;
	FT_DispHCycle =  408;
	FT_DispHOffset = 70;
	FT_DispHSync0 = 0;
	FT_DispHSync1 = 10;
	FT_DispVCycle = 263;
	FT_DispVOffset = 13;
	FT_DispVSync0 = 0;
	FT_DispVSync1 = 2;
	FT_DispPCLK = 8;
	FT_DispSwizzle = 2;
	FT_DispPCLKPol = 0;
#endif

    Ft_Gpu_Hal_Wr16(phost, REG_HCYCLE, FT_DispHCycle);
    Ft_Gpu_Hal_Wr16(phost, REG_HOFFSET, FT_DispHOffset);
    Ft_Gpu_Hal_Wr16(phost, REG_HSYNC0, FT_DispHSync0);
    Ft_Gpu_Hal_Wr16(phost, REG_HSYNC1, FT_DispHSync1);
    Ft_Gpu_Hal_Wr16(phost, REG_VCYCLE, FT_DispVCycle);
    Ft_Gpu_Hal_Wr16(phost, REG_VOFFSET, FT_DispVOffset);
    Ft_Gpu_Hal_Wr16(phost, REG_VSYNC0, FT_DispVSync0);
    Ft_Gpu_Hal_Wr16(phost, REG_VSYNC1, FT_DispVSync1);
    Ft_Gpu_Hal_Wr8(phost, REG_SWIZZLE, FT_DispSwizzle);
    Ft_Gpu_Hal_Wr8(phost, REG_PCLK_POL, FT_DispPCLKPol);
    Ft_Gpu_Hal_Wr8(phost, REG_PCLK,FT_DispPCLK);//after this display is visible on the LCD
    Ft_Gpu_Hal_Wr16(phost, REG_HSIZE, FT_DispWidth);
    Ft_Gpu_Hal_Wr16(phost, REG_VSIZE, FT_DispHeight);


    /* Touch configuration - configure the resistance value to 1200 - this value is specific to customer requirement and derived by experiment */
    Ft_Gpu_Hal_Wr16(phost, REG_TOUCH_RZTHRESH,1200);
    Ft_Gpu_Hal_Wr8(phost, REG_GPIO_DIR,0xff);
    Ft_Gpu_Hal_Wr8(phost, REG_GPIO,0x0ff);



}

#define SAMAPP_ENABLE_APIS_SET0
#ifdef SAMAPP_ENABLE_APIS_SET0

/* Optimized implementation of sin and cos table - precision is 16 bit */
FT_PROGMEM ft_prog_uint16_t sintab[] = {
  0, 402, 804, 1206, 1607, 2009, 2410, 2811, 3211, 3611, 4011, 4409, 4807, 5205, 5601, 5997, 6392, 
  6786, 7179, 7571, 7961, 8351, 8739, 9126, 9511, 9895, 10278, 10659, 11038, 11416, 11792, 12166, 12539,
  12909, 13278, 13645, 14009, 14372, 14732, 15090, 15446, 15799, 16150, 16499, 16845, 17189, 17530, 17868,
  18204, 18537, 18867, 19194, 19519, 19840, 20159, 20474, 20787, 21096, 21402, 21705, 22004, 22301, 22594, 
  22883, 23169, 23452, 23731, 24006, 24278, 24546, 24811, 25072, 25329, 25582, 25831, 26077, 26318, 26556, 26789, 
  27019, 27244, 27466, 27683, 27896, 28105, 28309, 28510, 28706, 28897, 29085, 29268, 29446, 29621, 29790, 29955, 
  30116, 30272, 30424, 30571, 30713, 30851, 30984, 31113, 31236, 31356, 31470, 31580, 31684, 31785, 31880, 31970, 
  32056, 32137, 32213, 32284, 32350, 32412, 32468, 32520, 32567, 32609, 32646, 32678, 32705, 32727, 32744, 32757, 
  32764, 32767, 32764};




ft_int16_t qsin(ft_uint16_t a)
{
  ft_uint8_t f;
  ft_int16_t s0,s1;

  if (a & 32768)
    return -qsin(a & 32767);
  if (a & 16384)
    a = 32768 - a;
  f = a & 127;
  s0 = ft_pgm_read_word(sintab + (a >> 7));
  s1 = ft_pgm_read_word(sintab + (a >> 7) + 1);
  return (s0 + ((ft_int32_t)f * (s1 - s0) >> 7));
}

/* cos funtion */
ft_int16_t qcos(ft_uint16_t a)
{
  return (qsin(a + 16384));
}
#endif



/*****************************************************************************/
/* Example code to display few points at various offsets with various colors */



/* Boot up for FT800 followed by graphics primitive sample cases */
/* Initial boot up DL - make the back ground green color */
const ft_uint8_t FT_DLCODE_BOOTUP[12] = 
{
  255,255,255,2,//GPU instruction CLEAR_COLOR_RGB
  7,0,0,38, //GPU instruction CLEAR
  0,0,0,0,  //GPU instruction DISPLAY
};



char *info[] = { 
					  "FT800 Restaurant Demo Application",
          			  "APP to demonstrate interactive Menu,",
					  "and animation,",	 
					  "using Bitmaps &Track",	 
					 };


 typedef struct logoim
{
  ft_prog_char8_t name[10];
  ft_uint16_t image_height;
  ft_uint8_t image_format;
  ft_uint8_t filter;
  ft_uint16_t sizex;
  ft_uint16_t sizey;
  ft_uint16_t linestride;
  ft_uint16_t gram_address;
}t_imageprp;



FT_PROGMEM t_imageprp Main_Icons[7] = {
                             {"Menu.BIN",  53,  RGB565,  BILINEAR,    114,   76,  80*2,  0L     },    //handle 0 
                             {"BB.BIN",    50,  ARGB4,   NEAREST,    50,    50,  50*2,  153},    //handle 1
                             {"HB.BIN",    50,  ARGB4,   NEAREST,    50,    50,  50*2,  159},    //handle 2
                             {"CB.BIN",    50,  ARGB4,   NEAREST,    50,    51,  51*2,  165},    //handle 3
                             {"OB.BIN",    50,  RGB565,   NEAREST,    50,    50,  50*2,  171},    //handle 4
                             {"PB.BIN",    26,  ARGB4,   NEAREST,    26,    26,  26*2,  177},    //handle 5
                             {"MB.BIN",    6,  ARGB4,   NEAREST,    26,    26,  26*2,  180},    //handle 6
			   };

void load_inflate_image(ft_uint32_t address, const char *filename)
{
	FILE *afile;
	ft_uint32_t ftsize=0;
	ft_uint8_t pbuff[8192];
	ft_uint16_t blocklen;

	Ft_Gpu_Hal_WrCmd32(phost,CMD_INFLATE);
	Ft_Gpu_Hal_WrCmd32(phost,address*1024);
	afile = fopen(filename,"rb");		// read Binary (rb)
	fseek(afile,0,SEEK_END);
	ftsize = ftell(afile);
	fseek(afile,0,SEEK_SET);	
	while(ftsize>0)
	{
		blocklen = ftsize>8192?8192:ftsize;
		fread(pbuff,1,blocklen,afile);							/* copy the data into pbuff and then transfter it to command buffer */			
		ftsize -= blocklen;
		Ft_Gpu_Hal_WrCmdBuf(phost,pbuff,blocklen);				/* copy data continuously into command memory */
	}
	fclose(afile);												/* close the opened jpg file */
}


void Logo_Intial_setup(struct logoim sptr[],ft_uint8_t num)
{
	ft_uint8_t z;
   for(z=0;z<num;z++)
   {
     load_inflate_image(sptr[z].gram_address,sptr[z].name);
   }     
   
   Ft_Gpu_CoCmd_Dlstart(phost);        // start
   Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
   for(z=0;z<num;z++)
   {
     Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(z));					// bg             
     Ft_App_WrCoCmd_Buffer(phost,BITMAP_SOURCE(sptr[z].gram_address*1024L));      
     Ft_App_WrCoCmd_Buffer(phost,BITMAP_LAYOUT(sptr[z].image_format,sptr[z].linestride,sptr[z].image_height));      
    //// if(Menu_prp==1 && z==1)          
    // Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(sptr[z].filter,REPEAT,BORDER,sptr[z].sizex,sptr[z].sizey)); 
    // else
     Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(sptr[z].filter,BORDER, BORDER, sptr[z].sizex,sptr[z].sizey)); 
   }
   Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
   Ft_Gpu_CoCmd_Swap(phost);
   Ft_App_Flush_Co_Buffer(phost);
   Ft_Gpu_Hal_WaitCmdfifo_empty(phost);    
}
float linear(float p1,float p2,ft_uint16_t t,ft_uint16_t rate)
{
	float st  = (float)t/rate;
	return p1+(st*(p2-p1));
}

ft_uint16_t smoothstep(ft_int16_t p1,ft_int16_t p2,ft_uint16_t t,ft_uint16_t rate)
{
	float dst  = (float)t/rate;
	float st = SQ(dst)*(3-2*dst);
	return p1+(st*(p2-p1));
}

ft_uint16_t acceleration(ft_uint16_t p1,ft_uint16_t p2,ft_uint16_t t,ft_uint16_t rate)
{
	float dst  = (float)t/rate;
	float st = SQ(dst);
	return p1+(st*(p2-p1));
}

float deceleration(ft_uint16_t p1,ft_uint16_t p2,ft_uint16_t t,ft_uint16_t rate)
{
	float st,dst  = (float)t/rate;
	dst = 1-dst;
	st = 1-SQ(dst);
	return p1+(st*(p2-p1));
}




static ft_uint8_t key_pressed = 0;

ft_uint8_t Read_Keys()
{
	static ft_uint8_t Read_tag=0,temp_tag=0,ret_tag=0;	
	ret_tag = NULL;		
	Read_tag = Ft_Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);						
	if(Read_tag!=NULL)											// Allow if the Key is released
	{
		if(temp_tag!=Read_tag)
		{
			temp_tag = Read_tag;	
			key_pressed = Read_tag;										// Load the Read tag to temp variable	
		}  
	}
	else
	{
		key_pressed = 0;
		if(temp_tag!=0)
		{
			ret_tag = temp_tag;
			temp_tag = 0;
		}  
	}
	return ret_tag;
}

static ft_uint8_t istouch()
{
  return !(Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_RAW_XY) & 0x8000);
}
void Use_Rgb()
{
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(1,1,1,1));	
}
void Use_Alpha()
{
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
}

void Clear_Alpha()
{
	Ft_App_WrCoCmd_Buffer(phost,COLOR_MASK(0,0,0,1));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(0));
}
void end_frame()
{
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
}

void Load_file2gram(ft_uint32_t add,ft_uint8_t sectors, FILE *afile)
{
  ft_uint8_t pbuff[512],temp[512],tval;
  ft_uint16_t z = 0;
  for(z=0;z<sectors;z++)
  {
	fread(pbuff,1,512,afile);
	Ft_Gpu_Hal_WrMem(phost,add,pbuff,512L);
	add+=512;
  }	
}

ft_void_t Play_Sound(ft_uint8_t sound,ft_uint8_t volume)
{
  Ft_Gpu_Hal_Wr8(phost,REG_VOL_SOUND,volume);
  Ft_Gpu_Hal_Wr8(phost,REG_SOUND,sound);
  Ft_Gpu_Hal_Wr8(phost,REG_PLAY,1);
}	

       
ft_void_t Info()
{
  ft_uint16_t dloffset = 0,z=0;
  Ft_CmdBuffer_Index = 0;
  Ft_Gpu_CoCmd_Dlstart(phost); 
  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
   Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight/2,28,OPT_CENTERX|OPT_CENTERY,"Please tap on a dot");
  Ft_Gpu_CoCmd_Calibrate(phost,0);
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  
  Ft_Gpu_CoCmd_Logo(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  while(0!=Ft_Gpu_Hal_Rd16(phost,REG_CMD_READ)); 
  dloffset = Ft_Gpu_Hal_Rd16(phost,REG_CMD_DL);
  dloffset -=4;
  Ft_Gpu_Hal_WrCmd32(phost,CMD_MEMCPY);
  Ft_Gpu_Hal_WrCmd32(phost,200*1024L);
  Ft_Gpu_Hal_WrCmd32(phost,RAM_DL);
  Ft_Gpu_Hal_WrCmd32(phost,dloffset);
  do
  {
    Ft_Gpu_CoCmd_Dlstart(phost);   
    Ft_Gpu_CoCmd_Append(phost,200*1024L,dloffset);
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_B(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_C(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_D(0));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(256));
    Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_F(0));  
    Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());	
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(219,180,150));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(220));
    Ft_App_WrCoCmd_Buffer(phost,BEGIN(EDGE_STRIP_A));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(0,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(FT_DispWidth*16,FT_DispHeight*16));
    Ft_App_WrCoCmd_Buffer(phost,COLOR_A(255));
    Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());	
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
   // INFORMATION 
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,20,28,OPT_CENTERX|OPT_CENTERY,info[0]);
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,60,26,OPT_CENTERX|OPT_CENTERY,info[1]);
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,90,26,OPT_CENTERX|OPT_CENTERY,info[2]);  
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,120,26,OPT_CENTERX|OPT_CENTERY,info[3]);  
    Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2,FT_DispHeight-30,26,OPT_CENTERX|OPT_CENTERY,"Click to play");
    if(key_pressed!='P')
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
    else
    Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(100,100,100));


	Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));   
    Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(20*16));
    Ft_App_WrCoCmd_Buffer(phost,TAG('P'));
    Ft_App_WrCoCmd_Buffer(phost,VERTEX2F((FT_DispWidth/2)*16,(FT_DispHeight-60)*16));

    Ft_App_WrCoCmd_Buffer(phost,CLEAR(0,1,0));

    Ft_App_WrCoCmd_Buffer(phost, COLOR_RGB(255, 0, 0) );
    Ft_App_WrCoCmd_Buffer(phost, STENCIL_OP(INCR,INCR) );
    Ft_App_WrCoCmd_Buffer(phost, COLOR_MASK(0,0,0,0) );//mask all the colors
    Ft_App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_L));

    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-77+5),0,0));
    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-2+15),(FT_DispHeight-67+8),0,0));
    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-57+11),0,0));
    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-77+5),0,0));

    Ft_App_WrCoCmd_Buffer(phost, COLOR_MASK(1,1,1,1) );//enable all the colors
    Ft_App_WrCoCmd_Buffer(phost, STENCIL_FUNC(EQUAL,1,255));
    Ft_App_WrCoCmd_Buffer(phost, BEGIN(EDGE_STRIP_L));
    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(FT_DispWidth,0,0,0));
    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(FT_DispWidth,FT_DispHeight,0,0));
	
    Ft_App_WrCoCmd_Buffer(phost, STENCIL_OP(KEEP,KEEP));
    Ft_App_WrCoCmd_Buffer(phost, STENCIL_FUNC(ALWAYS,0,255));
    Ft_App_WrCoCmd_Buffer(phost,LINE_WIDTH(1*16));
	
    Ft_App_WrCoCmd_Buffer(phost, BEGIN(LINE_STRIP));
    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-77+5),0,0));
    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-2+15),(FT_DispHeight-67+8),0,0));
    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-57+11),0,0));
    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(((FT_DispWidth/2)-14+10),(FT_DispHeight-77+5),0,0));
	
    Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
    Ft_Gpu_CoCmd_Swap(phost);
    Ft_App_Flush_Co_Buffer(phost);
    Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
	
  }while(Read_Keys()!='P');

  Ft_Gpu_CoCmd_Dlstart(phost);   
  Ft_App_WrCoCmd_Buffer(phost,CLEAR_COLOR_RGB(255,255,255));
  Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));   
  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
  Ft_Gpu_CoCmd_Text(phost,FT_DispWidth/2-80,FT_DispHeight/2,29,OPT_CENTERX|OPT_CENTERY,"Data Loading");
  Ft_Gpu_CoCmd_Spinner(phost,FT_DispWidth/2+40,FT_DispHeight/2,1,0);
  Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
  Ft_Gpu_CoCmd_Swap(phost);
  Ft_App_Flush_Co_Buffer(phost);
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
  Play_Sound(0x50,255);
}


//static ft_int16_t trackval = 0;

static float trackval = 0;

ft_uint8_t description_no = 0;

ft_int32_t StoreTag[16],CheckTag[16];
ft_uint8_t item_selection = 0;
ft_uint8_t item_selected_counts = 0;

char *Array_Beverage_name[5] = {0,"Espresso", "Black Coffee", "Hot Choclate","Tea"};
char *Array_Frappe_name[5] = {0,"Walnut", "Choclate", "Strawberry","Vanilla"};
char *Array_Pastry_name[6] = {0, "Blueberry Muffin", "Choc Muffin", "Choc Donut", "Pain au Choclat", "Tiramisu"};
char *Array_MainMenu_name[4] = {0, "Beverages", "Pastries", "Frappe"};



ft_int16_t Touch;
ft_uint8_t item_no = 0;

void Back_Home(ft_uint8_t alpha,ft_uint8_t OrderFlag)
{
	ft_uint8_t TagValue=0;
	
	TagValue = Ft_Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);
	Touch =  Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);
  Ft_App_WrCoCmd_Buffer(phost,COLOR_A(0));
   Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(1));
   Ft_App_WrCoCmd_Buffer(phost, TAG('T'));
   Ft_Gpu_CoCmd_Button(phost,10,200,470,70,0,0,"");
   Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());  
   Ft_App_WrCoCmd_Buffer(phost,COLOR_A(alpha));  

   Ft_App_WrCoCmd_Buffer(phost, TAG('B'));
   Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(10,220,1,0));
   Ft_App_WrCoCmd_Buffer(phost, TAG('H'));
   Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(70,220,2,0));
	   if(OrderFlag ==0)
	   {
	   Ft_App_WrCoCmd_Buffer(phost, TAG('O'));
	   Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));						
	   Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(430,220,4,0));	
	   }
   Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());  
    Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(0));

}

ft_uint8_t Selection_Menu(ft_uint8_t NoItems,ft_uint8_t start_tag)
{ 
	float zinout_f = 0;
  ft_int32_t change,noitems,prevth=0,currth=0;
  ft_int16_t theta,zinout,scroller_vel=0;
  ft_uint8_t Menu=0,temp_Menu=0;
  ft_uint8_t x,rotation_flag=0,flag=0;
  ft_int16_t key_deteted_cts=0,xoff,yoff,rotation_cts=0,deg = 0; 
  
  Ft_CmdBuffer_Index = 0;
  Ft_Gpu_CoCmd_Track(phost,240,0,1,1,1);
  Ft_Gpu_CoCmd_Track(phost,240,0,1,1,2);
  Ft_Gpu_CoCmd_Track(phost,240,0,1,1,3);
  Ft_Gpu_CoCmd_Track(phost,240,0,1,1,4);
  Ft_App_Flush_Co_Buffer(phost);													//L8 format
  Ft_Gpu_Hal_WaitCmdfifo_empty(phost);

  deg = 360/6;
 
  do
  {
	 
     ft_uint32_t sy = Ft_Gpu_Hal_Rd32(phost,REG_TRACKER);
     ft_uint8_t key_detected = sy&0xff;    
	  ft_int16_t TagCheck = Ft_Gpu_Hal_Rd16(phost, REG_TOUCH_SCREEN_XY);


     if(key_detected>0)
     { 
       key_deteted_cts++;
       if(key_detected<4)
	   temp_Menu = key_detected; else temp_Menu = 0;
	   Menu = 0;			
	   rotation_cts = 0;
     }   
     if(key_detected==0)
     {
	   
       if(key_deteted_cts>0&&key_deteted_cts<15)
	   {
			if(!rotation_flag)
				Menu = temp_Menu; else rotation_flag = 0;
	   }			
       key_deteted_cts = 0;
	   if(rotation_cts>100)
	   {	
		   trackval+=0.5; 		
			rotation_flag = 1;
	   }	
	   else 
       rotation_cts++;
     }

      if(key_deteted_cts>15)
     {  
			 float tempval;
			 currth = (sy>>16);
			 tempval = ((currth-prevth));
		 
			 if((currth-prevth) > 32000)
			 {
				 tempval = 65536 - currth + prevth;
			 }
			 else if((currth-prevth) < -32000)
			 {
				 tempval = 65536 - prevth + currth;
				 tempval = -tempval;
			 }
			 tempval = (tempval*360)/65536;
			 rotation_flag = 0;
		   

		   if(prevth!=0)
		   {
				trackval += tempval;
		   }
		   prevth = currth;
		   rotation_cts = 0;
     }
	  else
     {
        prevth=0;    
     }  
     
	Ft_Gpu_CoCmd_Dlstart(phost);        // start
	Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
	Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
	Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(1));
	Ft_App_WrCoCmd_Buffer(phost, TAG(4));	
	Ft_Gpu_CoCmd_Gradient(phost,0, 0, 0x000000, 0, 180, 0xFF0000);
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(50));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
	Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(250*16));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F((240)*16,-150));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_A(60));
	Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0));
	Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(135*16));
	Ft_App_WrCoCmd_Buffer(phost, VERTEX2F((240)*16,-150));
	Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
		Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(0));
	Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));

	
	
    for(x=1;x<=NoItems;x++)
	{
		float tempval;
		theta = deg*x;		
		tempval = deg*x + trackval+deg;

		theta = (ft_int16_t)((ft_int32_t)tempval%220); 
		
		xoff = 200+(ft_int16_t)(190*cos(theta*0.0174));
		yoff = -20+(ft_int16_t)(150*sin(theta*0.0174)); 
		zinout = 310 - yoff;

  	    Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(BILINEAR, BORDER, BORDER, (ft_int32_t)(256*80/zinout), (ft_int32_t)(256*53/zinout)  ));
        Ft_App_WrCoCmd_Buffer(phost,CELL(x-1));
		
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(zinout));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(zinout)); 
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		Ft_App_WrCoCmd_Buffer(phost,TAG(x));	
		Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(xoff*16,yoff*16));
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

   }
   Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
   Ft_Gpu_CoCmd_Text(phost,150,50,26,0,"Please select one of the Options");
   for(x=1;x<=NoItems;x++)
	{
		float tempval1 ;
		theta = deg*x;		
		tempval1 = deg*x + trackval+deg;

		theta = (ft_int16_t)((ft_int32_t)tempval1%220); 

		xoff = 200+(ft_int16_t)(190*cos(theta*0.0174));
		yoff = -20+(ft_int16_t)(150*sin(theta*0.0174)); 
		zinout_f  =  310-yoff;
 		zinout_f  =  255/zinout_f;
		zinout = 5+yoff+(ft_int16_t)(53*zinout_f);
		Ft_Gpu_CoCmd_Text(phost,xoff+5,(zinout),16,OPT_CENTERY,Array_MainMenu_name[x]);
   }
  
   Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
   Ft_Gpu_CoCmd_Swap(phost);
   Ft_App_Flush_Co_Buffer(phost);
   Ft_Gpu_Hal_WaitCmdfifo_empty(phost);       

  }while(Menu==0);
  return Menu;  
}






static struct {
  signed short dragprev;
  int vel;      // velocity
  long base;    // screen x coordinate, in 1/16ths pixel
  long limit;
} scroller;

static void scroller_init(ft_uint32_t limit)
{
  scroller.dragprev = -32768;
  scroller.vel = 0;      // velocity
  scroller.base = 0;     // screen x coordinate, in 1/16ths pixel
  scroller.limit = limit;
}
static void scroller_run()
{
  signed short sy = Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);
  
  if ((sy != -32768) & (scroller.dragprev != -32768)) {
    scroller.vel = (scroller.dragprev - sy) << 4;
  } else {
    int change = max(1, abs(scroller.vel) >> 5);
    if (scroller.vel < 0)
      scroller.vel += change;
    if (scroller.vel > 0)
      scroller.vel -= change;
  }
  scroller.dragprev = sy;
  
  scroller.base += scroller.vel;
  
  scroller.base = max(0, min(scroller.base, scroller.limit));
}

typedef struct Item_Prop
{
	char *ItemName;
	char *ItemPrice;
	char *ItemDescription;
	ft_int32_t ItemPriceinVal;
}Item_Properties;

Item_Properties Item_Property[] = 
{
	{0,0,0},//0

	{"Espresso", "$3.00", "Italian Espresso Coffee",300},//1
	{"Black Coffee", "$2.00", "Freshly Brewed Coffee",200},//2
	{"Hot Choclate", "$3.00", "The Finest cocoa blended with hot milk",300},//3
	{"Tea", "$2.00", "Hot Tea",200},//4
	{"Blueberry Muffin", "$2.00", "A moist, flavorful muffin packed with juicy blueberries",200},//5
	{"Choclate Muffin", "$2.00", "A delicious muffin packed with rich choclate",200},//6
	{"Choclate Donut", "$2.00", "A classic glazed chocolate doughnut",200},//7
	{"Pain au Choclat", "$2.00", "A light, flaky pastry with chocolate",200},//8
	{"Tiramisu", "$2.00", "Creamy traditional tiramisu",200},//9
	{"Walnut Frappe", "$3.00","Coffee blended with Walnut drizzle",300},//10
	{"Choclate Frappe", "$3.00","A creamy blend of chocolaty chips, milk and ice",300},//11
	{"Strawberry Frappe", "$3.00","Strawberries and milk blended with ice",300},//12
	{"Vanilla Frappe", "$3.00","A rich and creamy blend of vanilla bean, milk and ice",300},//13
};


ft_uint8_t Order_list()
{
	ft_uint8_t dg_count=0,i=0,Key_counts=0,OrderFlag=0,BHFlag=0;
	ft_uint8_t  z,Read_tag,temp = 0,TagVal,y=0;
	ft_int16_t sy = 0,	drag = 0,PlusTag = 101,MinusTag = 201,QtyNo=0,TotPrice=0,TotItems=0,PosY=0;
	ft_int16_t yoff = 40,alpha =255,IconCt=0;
	
	scroller_init((FT_DispHeight*2)*16);

	do
	{
			sy =  Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);
			TagVal = Ft_Gpu_Hal_Rd8(phost,REG_TOUCH_TAG);


			Read_tag = Read_Keys();
			if(TagVal != 0)
				Key_counts++;
			if(sy != -32768)	
			{
			  dg_count++;     	
			  temp = Read_tag;      
			}
			/*Move into the particular frame based on dragdt now 30pixels*/
			if(sy==-32768)
			{
			  dg_count = 0;			 
			  drag = scroller.base>>4; 	
			}
			/*if tag in but still pendown take a scroller basevalue*/
			else if(dg_count>15)
			{
			  Read_tag = 0;
			  temp = Read_tag;
			  drag = scroller.base>>4;
			}
			if(drag >=282)drag = 282;
			if(Read_tag !='H' && Read_tag !='B'&&drag <= 282&& Key_counts > 15)
			{
				scroller_run();
			//	Key_counts =0;
			}
			Ft_Gpu_CoCmd_Dlstart(phost);        // start
			Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));	
			Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());	
		
			Ft_Gpu_CoCmd_Gradient(phost,0, 0, 0x000000, 0, 180, 0xFF0000);
		
			yoff = 50;
			yoff-=drag;
			TotPrice =0;
			TotItems=0;
			for(z=1,y=1;y<16,z<15;z++)
			{

				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));			
				 PosY = yoff+(y-1)*30; 
			
				if((CheckTag[z]!=0) && (StoreTag[z]>=0))
				{
					   
						Ft_Gpu_CoCmd_Text(phost,20,PosY,27,OPT_CENTERY,Item_Property[z].ItemName);
						
						if((Read_tag>0)&&(Read_tag == z))
						{
							Play_Sound(0x56,255);
							StoreTag[z]++;
							Read_tag = 0;
							Key_counts = 0;
						}
						if((Read_tag>0)&&(Read_tag==z+30))
						{
							Play_Sound(0x56,255);
							StoreTag[z]--;
							Read_tag = 0;
							Key_counts =0;
						}
						if(StoreTag[z] < 0)StoreTag[z] = 0;
						Ft_Gpu_CoCmd_Number(phost,210,PosY,27,OPT_CENTERY,StoreTag[z]);
						TotItems+= StoreTag[z];
				
						Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
						Ft_App_WrCoCmd_Buffer(phost,TAG(z));
						Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(165,PosY-18,30,'+'));

						Ft_App_WrCoCmd_Buffer(phost,TAG(z+30));

						Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(245,PosY-25,31,'-'));	
				
						Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

						Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
						Ft_Gpu_CoCmd_Number(phost,340,PosY,27,OPT_CENTERY,StoreTag[z]*(Item_Property[z].ItemPriceinVal/100));

						TotPrice+=StoreTag[z]*(Item_Property[z].ItemPriceinVal/100);
						y++;
					
				}				
		}		
		 // to reduce points in the cart if the total number of items is less than 15
		if(TotItems < 15) 
		{
			item_selected_counts = TotItems;
		}
		else if(TotItems > 15) 
			item_selected_counts = 15;
		Ft_Gpu_CoCmd_Text(phost,20,yoff+(y-1)*30,29,OPT_CENTERY,"Total");
		Ft_Gpu_CoCmd_Number(phost,340,yoff+(y-1)*30,29,OPT_CENTERY,TotPrice);
		
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(150,150,150));
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(10*16,5*16));
		Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(460*16,30*16));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
		Ft_Gpu_CoCmd_Text(phost,30,15,27,OPT_CENTERY,"Item");
		Ft_Gpu_CoCmd_Text(phost,190,15,27,OPT_CENTERY,"Quantity");
		Ft_Gpu_CoCmd_Text(phost,340,15,27,OPT_CENTERY,"Price($)");

		Ft_App_WrCoCmd_Buffer(phost, BEGIN(BITMAPS));
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
        Ft_App_WrCoCmd_Buffer(phost,COLOR_A(0));
  
		Ft_App_WrCoCmd_Buffer(phost, TAG('T'));
		Ft_Gpu_CoCmd_Button(phost,10,200,470,70,0,0,"");
		Ft_App_WrCoCmd_Buffer(phost, RESTORE_CONTEXT());
   		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
		  if(Read_tag == 'T'||IconCt ==0)
           {
			   BHFlag =1;
			   alpha = 255;
		  }
		  if(BHFlag == 1)
		  {
				if((alpha>0)&&(IconCt >= 150)&&(sy== -32768)) 
					alpha--;
	  			Back_Home(alpha,1);
		   }

		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		 
		Ft_App_WrCoCmd_Buffer(phost, RESTORE_CONTEXT());
		Ft_App_WrCoCmd_Buffer(phost, TAG('P'));
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
		Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(22*16));
	    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(430,243,0,0));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
		Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(16*16));
	    Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(430,243,0,0));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
		Ft_Gpu_CoCmd_Text(phost,423,242,29,OPT_CENTERY,"$");
		Ft_App_WrCoCmd_Buffer(phost,END());
	    Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));

		if(Read_tag == 'P' && Key_counts >15)i=1;
		if((i>0)&&( i <100))
		{
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
				Ft_App_WrCoCmd_Buffer(phost,BEGIN(RECTS));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(110*16,115*16));
				Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(410*16,180*16));
				Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));
				Ft_Gpu_CoCmd_Text(phost,120,125,27,OPT_CENTERY,"Please make payment at the counter.");
				Ft_Gpu_CoCmd_Text(phost,200,150,27,OPT_CENTERY,"Thank You.");	
		}
		if(i>=100) i =0;
		Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
		Ft_Gpu_CoCmd_Swap(phost);
		Ft_App_Flush_Co_Buffer(phost);
		Ft_Gpu_Hal_WaitCmdfifo_empty(phost);      
		if(i>=1)i++;
		IconCt++;
		if(Read_tag == 'P'||Read_tag == 255) Read_tag=0;
	}while((Read_tag==0)||(Read_tag < 50 )|| Read_tag== 'T');
if(Read_tag == 'H')	 Play_Sound(0x51,255);
return	Read_tag ;

}
ft_uint8_t ItemSelection(ft_uint8_t NoItems,ft_uint8_t start_cell,char **Menu_items)
{ 
	float zinout_f=0;
	
	ft_uint8_t Menu=0,temp_Menu=0,x,rotation_flag=0,StCell=0,Read_tag,CartFlag=0,i,TouchTag=0,BHFlag=0;
	ft_int16_t prevth=0,currth=0,key_deteted_cts=0,xoff,yoff,rotation_cts=0,deg = 0,theta,zinout,scroller_vel=0,sx,sy,alpha = 255,Touch; 
	ft_int32_t change,k,points_cdsX[16],points_cdsY[16];	
	ft_uint32_t IconCt=0;	

	Ft_CmdBuffer_Index = 0;
	Ft_Gpu_CoCmd_Track(phost,240,0,1,1,1);
	Ft_Gpu_CoCmd_Track(phost,240,0,1,1,2);
	Ft_Gpu_CoCmd_Track(phost,240,0,1,1,3);
	Ft_Gpu_CoCmd_Track(phost,240,0,1,1,4);
	Ft_Gpu_CoCmd_Track(phost,240,0,1,1,5);

	// for gradient
	Ft_Gpu_CoCmd_Track(phost,240,0,1,1,90);
  
	Ft_App_Flush_Co_Buffer(phost);													//L8 format
	Ft_Gpu_Hal_WaitCmdfifo_empty(phost);
	deg = 360/6;

  	points_cdsX[0] = 247;
	
	for(k=1; k<=15;k++)
	{
		  points_cdsX[k] = points_cdsX[k-1] -5;		
		  points_cdsX[0] = 248;
		  points_cdsX[6] = 248;
		  points_cdsX[11] = 248;
	}

	for(k=1; k<=15;k++)
	{
		if(k < 6)	points_cdsY[k] =  68;
		if((k >= 6) && (k < 11))	points_cdsY[k] =  64;
		if((k >= 11) && (k < 16))points_cdsY[k] =  60;
	}

	
	  StCell = start_cell;
	do
	{
	    
     ft_uint32_t ssy = Ft_Gpu_Hal_Rd32(phost,REG_TRACKER);
     ft_uint8_t key_detected = ssy&0xff;   
	 TouchTag = Ft_Gpu_Hal_Rd8(phost, REG_TOUCH_TAG);
	 Read_tag = Read_Keys();
	 Touch =  Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);
	 
	 Ft_Gpu_CoCmd_Dlstart(phost);        // start
	 Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
	 Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());
	
	
	 Ft_Gpu_CoCmd_Gradient(phost,0, 0, 0x000000, 0, 180, 0xFF0000);
	 Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));
	 Ft_App_WrCoCmd_Buffer(phost,COLOR_A(50));
	 Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,0,0));
	 Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(1));
	 Ft_App_WrCoCmd_Buffer(phost, TAG(90));
	 Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(250*16));
	 Ft_App_WrCoCmd_Buffer(phost, VERTEX2F((240)*16,-150));
	 Ft_App_WrCoCmd_Buffer(phost,COLOR_A(60));
	 Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,0,0));
	 
	 Ft_App_WrCoCmd_Buffer(phost,TAG(91));
	 Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(135*16));
	 Ft_App_WrCoCmd_Buffer(phost, VERTEX2F((240)*16,-150));
	 Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
	 Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
	

	 if(key_pressed > 0 && key_pressed < 20)
	 	  description_no = key_pressed+item_no;
	


     if(key_detected>0 && key_detected!='T')
     { 
       key_deteted_cts++;
       if(key_detected<NoItems+1)
	   temp_Menu = key_detected; 
	   else
		   temp_Menu = 0;
	   Menu = 0;			
	   rotation_cts = 0;
     }   
     if(key_detected==0)
     {	   
       if(key_deteted_cts>=0&&key_deteted_cts<15)
	   {
			if(!rotation_flag)
				Menu = temp_Menu; else rotation_flag = 0;
	   }			
       key_deteted_cts = 0;
	   if(rotation_cts>100)
	   {	
			trackval+=0.5; 
			item_selection=0;
			rotation_flag = 1;
	   }	
	   else 
		 rotation_cts++;
     }     
	 
	   if(key_deteted_cts>15 && item_selection==0&&TouchTag!=91)
     {  
			 float tempval;
			 currth = (ssy>>16);
			 tempval = ((currth-prevth));
		 
			 if((currth-prevth) > 32000)
			 {
				 tempval = 65536 - currth + prevth;
			 }
			 else if((currth-prevth) < -32000)
			 {
				 tempval = 65536 - prevth + currth;
				 tempval = -tempval;
			 }
			 tempval = (tempval*360)/65536;
			 rotation_flag = 0;
		 
		   if(prevth!=0)
		   {
				 trackval += tempval;
		   }
		   prevth = currth;
		   rotation_cts = 0;
     }
	  else
     {
        prevth=0;    
     }  
     
    if(key_detected=='T')
	{
			alpha = 255;
			IconCt = 0;
	}	
	if(key_pressed >0 && key_pressed <6)
		i = key_pressed;
	if(Touch == -32768)
		i=0;
	if(i!=0 && key_pressed == 'S')
		item_selection=1;
	else
		item_selection =0;

	if(item_selection==1 && key_pressed=='S')
	 {
		 item_selection = 0;		 
		 Play_Sound(0x54,255);
		 if(item_selected_counts<17)
		 {
				 StoreTag[description_no-1]++;
				 CheckTag[description_no-1] =1;
	             item_selected_counts++;				 
		 }
		 key_pressed = 0;
	 }

	
	

	

	Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
	
	start_cell = StCell;
    for(x=1;x<=NoItems;x++)
	{

				float tempval;
		theta = deg*x;		
		tempval = deg*x + trackval+deg;
			if(NoItems == 4)
		theta = (ft_int16_t)((ft_int32_t)tempval%250); 
			else
		theta = (ft_int16_t)((ft_int32_t)tempval%300); 
		xoff = 200+(ft_int16_t)(190*cos(theta*0.0174));
		yoff = -20+(ft_int16_t)(150*sin(theta*0.0174)); 


		zinout = 310- yoff;

	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));              

        Ft_App_WrCoCmd_Buffer(phost,CELL(start_cell));

		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		Ft_App_WrCoCmd_Buffer(phost,TAG(x));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(zinout ));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(zinout ));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_SIZE(BILINEAR, BORDER, BORDER, 256*80/zinout, 256*53/zinout  ));
		Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(xoff*16,yoff*16));
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
		start_cell++;  
   }
	
		 Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());

    for(x=1;x<=NoItems;x++)
	{
		float tempval1 ;
		theta = deg*x;		
		tempval1 = deg*x + trackval+deg;
		if(NoItems == 4)
		theta = (ft_int16_t)((ft_int32_t)tempval1%250); 
		else
			theta = (ft_int16_t)((ft_int32_t)tempval1%300); 
		

		xoff = 200+(ft_int16_t)(190*cos(theta*0.0174));
		yoff = -20+(ft_int16_t)(150*sin(theta*0.0174)); 

		zinout_f  =  310-yoff;
 		zinout_f  =  255/zinout_f;
		zinout = 5+yoff+(ft_int16_t)(53*zinout_f);
		Ft_Gpu_CoCmd_Text(phost,xoff+5,zinout,16,OPT_CENTERY,Menu_items[x]);
    }
  
		Ft_App_WrCoCmd_Buffer(phost, BEGIN(BITMAPS));
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		Ft_App_WrCoCmd_Buffer(phost,COLOR_A(0));
  
	   Ft_App_WrCoCmd_Buffer(phost, TAG('T'));
	   Ft_Gpu_CoCmd_Button(phost,10,200,470,70,0,0,"");
	   Ft_App_WrCoCmd_Buffer(phost, RESTORE_CONTEXT());
   	   Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
		if(Read_tag == 'T'||IconCt ==0)
        {
			   BHFlag =1;
			   alpha = 255;
		}
	   if(BHFlag == 1)
	   {
				if((alpha>0)&&(IconCt > 150)&&(Touch== -32768)) 
					alpha--;
	  			Back_Home(alpha,0);
		}
		sx = -10;
		if((temp_Menu >0)&&(temp_Menu < 6))
		{
			   Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,255,0));
			   Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));		
			   Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(5*16));
			   sx =  Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY+2);
			   sy =  Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);
			   if(sx != -32768)
			   Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(sx*16,sy*16));			 
		}
		if(item_selected_counts >=16)item_selected_counts = 15;
		 if(item_selected_counts>0)
		{
			for(k=1;k<=item_selected_counts;k++)
			{
				  Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(0,255,0));
				  Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS));		
				  Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(3*16));		
				  Ft_App_WrCoCmd_Buffer(phost,VERTEX2F(points_cdsX[k]*16,points_cdsY[k]*16) );	  		  
			}
		}

		 Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS)); 
		 if((temp_Menu >0)&&(temp_Menu < 6))
			 Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,0));  
		else
			Ft_App_WrCoCmd_Buffer(phost,COLOR_RGB(255,255,255));  
   
	   Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(CartX,40,3,0));  
	   Ft_App_WrCoCmd_Buffer(phost,BEGIN(FTPOINTS)); 
	   Ft_App_WrCoCmd_Buffer(phost,POINT_SIZE(20*16)); 
	   Ft_App_WrCoCmd_Buffer(phost,COLOR_A(0));  	 
	   Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(1));
	   Ft_App_WrCoCmd_Buffer(phost, TAG('S'));
	   Ft_App_WrCoCmd_Buffer(phost, VERTEX2F(230*16,50*16));
	   Ft_App_WrCoCmd_Buffer(phost, TAG_MASK(0));
	   Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
	   Ft_Gpu_CoCmd_Swap(phost);
	   Ft_App_Flush_Co_Buffer(phost);
	   Ft_Gpu_Hal_WaitCmdfifo_empty(phost);       
   
	   if((CartX> 215)&& (CartFlag==0)) CartX-=5;
	   if(CartX == 215) CartFlag = 1;
	   if(Read_tag == 90) Read_tag=0;
	   if(key_deteted_cts > 15) 
	   {
		   Menu =0;
		   temp_Menu = 0;
	   }
	   if((Read_tag == 'B') || (Read_tag == 'H'))  
	   {
			 Play_Sound(0x51,255);
		   Menu = 'H';
	   }
		if(Read_tag == 'O')  Menu = 'O';
		if(Menu!=0) return Menu;
		IconCt++;
  }
  while(1);
  i=0;

  return Menu;

}


ft_uint8_t Description(ft_uint8_t description,ft_uint8_t item,ft_uint8_t Noitems)
{
	ft_uint8_t  Read_tag = 0,i=1,BHFlag=0;
	ft_uint16_t alpha = 255,IconCt=0;
    do
    {
		Touch =  Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);
		Read_tag = Read_Keys();
		Ft_Gpu_CoCmd_Dlstart(phost);        // start
		Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));	
		Ft_App_WrCoCmd_Buffer(phost,SAVE_CONTEXT());	
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_HANDLE(0));
			Ft_App_WrCoCmd_Buffer(phost,CELL(description+1));
		Ft_App_WrCoCmd_Buffer(phost, BITMAP_SIZE(BILINEAR,BORDER,BORDER,imw*2,imh*2));		
	
		Ft_Gpu_CoCmd_Gradient(phost,0, 0, 0x000000, 0, 180, 0xFF0000);
		Ft_App_WrCoCmd_Buffer(phost,BEGIN(BITMAPS));
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_A(128) );
		Ft_App_WrCoCmd_Buffer(phost,BITMAP_TRANSFORM_E(128) );
		Ft_App_WrCoCmd_Buffer(phost, VERTEX2II(30,80,0,description+1));
		Ft_App_WrCoCmd_Buffer(phost,RESTORE_CONTEXT());
		Ft_Gpu_CoCmd_Text(phost,30,30,25,0,Item_Property[description-1].ItemName);		
		Ft_Gpu_CoCmd_Text(phost,370,85,21,0,Item_Property[description-1].ItemPrice);		
   		Ft_Gpu_CoCmd_Text(phost,30,190,18,0,Item_Property[description-1].ItemDescription);			

		Ft_App_WrCoCmd_Buffer(phost, BEGIN(BITMAPS));
		Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(1));
		  Ft_App_WrCoCmd_Buffer(phost,COLOR_A(0));
  
   Ft_App_WrCoCmd_Buffer(phost, TAG('T'));
   Ft_Gpu_CoCmd_Button(phost,10,200,470,70,0,0,"");
   Ft_App_WrCoCmd_Buffer(phost, RESTORE_CONTEXT());
   	Ft_App_WrCoCmd_Buffer(phost,TAG_MASK(0));
		  if(Read_tag == 'T'||IconCt ==0)
           {
			   BHFlag =1;
			   alpha = 255;
		  }
		  if(BHFlag == 1)
		  {
		if((alpha>0)&&(IconCt > 150)&&(Touch== -32768)) 
			alpha--;
	  	Back_Home(alpha,0);
		   }

		    Ft_App_WrCoCmd_Buffer(phost,DISPLAY());
    Ft_Gpu_CoCmd_Swap(phost);
    Ft_App_Flush_Co_Buffer(phost);
    Ft_Gpu_Hal_WaitCmdfifo_empty(phost);    
	IconCt++;

    }	
    while(Read_tag==0 || Read_tag == 'T'||Read_tag  == 255);    
	if(Read_tag == 'H') Play_Sound(0x51,255);
    return Read_tag; 
}

#ifdef MSVC_PLATFORM
/* Main entry point */
ft_int32_t main(ft_int32_t argc,ft_char8_t *argv[])
#endif
#if defined(ARDUINO_PLATFORM) || defined(MSVC_FT800EMU)
ft_void_t setup()
#endif
{
	ft_uint8_t Menu;
	char *path = "..\\..\\..\\..\\Test";
	Ft_Gpu_HalInit_t halinit;
	
	halinit.TotalChannelNum = 1;

              
	Ft_Gpu_Hal_Init(&halinit);
	host.hal_config.channel_no = 0;
#ifdef MSVC_PLATFORM_SPI
	host.hal_config.spi_clockrate_khz = 12000; //in KHz
#endif
#ifdef ARDUINO_PLATFORM_SPI
	host.hal_config.spi_clockrate_khz = 4000; //in KHz
#endif
        Ft_Gpu_Hal_Open(&host);
            
	phost = &host;

    Ft_BootupConfig();
	printf("reg_touch_rz =0x%x ", Ft_Gpu_Hal_Rd16(phost, REG_TOUCH_RZ));
	printf("reg_touch_rzthresh =0x%x ", Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_RZTHRESH));
	printf("reg_touch_tag_xy=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG_XY));
	printf("reg_touch_tag=0x%x",Ft_Gpu_Hal_Rd32(phost, REG_TOUCH_TAG));


    /*It is optional to clear the screen here*/	
    Ft_Gpu_Hal_WrMem(phost, RAM_DL,(ft_uint8_t *)FT_DLCODE_BOOTUP,sizeof(FT_DLCODE_BOOTUP));
	Ft_Gpu_Hal_Wr8(phost, REG_DLSWAP,DLSWAP_FRAME);

  chdir(path);

  Info();
  trackval = 0;
  Logo_Intial_setup(Main_Icons,7);
  while(1)
  {
        ft_uint8_t exit = 0,NoItems=0;
	
		Menu = Selection_Menu(3,1); 
	
 	  do
	  {
		   Touch = Ft_Gpu_Hal_Rd16(phost,REG_TOUCH_SCREEN_XY);
		
		    item_selection = 0;
			Play_Sound(0x51,255);
		  do
		  {
			Ft_Gpu_CoCmd_Dlstart(phost);        // start
			Ft_App_WrCoCmd_Buffer(phost,CLEAR(1,1,1));
						
			switch(Menu)
			{
			  case 1:	
				Ft_Gpu_CoCmd_Swap(phost);
				Ft_App_Flush_Co_Buffer(phost);
				Ft_Gpu_Hal_WaitCmdfifo_empty(phost);     
				item_no =1;
				NoItems = 4;
				exit = ItemSelection(NoItems,3,Array_Beverage_name);   
			    break;
          
			  case 2: 
				Ft_Gpu_CoCmd_Swap(phost);
				Ft_App_Flush_Co_Buffer(phost);
				Ft_Gpu_Hal_WaitCmdfifo_empty(phost); 
				item_no =5;
				NoItems =5;
				exit = ItemSelection(NoItems,7,Array_Pastry_name);    
				break;
          
			  case 3: 
				Ft_Gpu_CoCmd_Swap(phost);
				Ft_App_Flush_Co_Buffer(phost);
				Ft_Gpu_Hal_WaitCmdfifo_empty(phost);       
				item_no =10;
				NoItems=4;
				exit = ItemSelection(NoItems,12,Array_Frappe_name);  
			    break;				
			} 

		  }while(exit==0); 
		 
			
		   if((exit >0)&& (exit <6) && (Touch == -32768))
		  {					  
				description_no = exit+item_no;
				if(description_no < 80)
				{
						Play_Sound(0x51,255);
						exit= Description(description_no,exit,item_no);
				}
		  }	                                                                                                        
		  
		   if(exit == 'O')
		  {			
				Play_Sound(0x51,255);
				exit = Order_list();
		   }
	  }while(exit!='H');
 
  }
  
    Ft_Gpu_Hal_Close(phost);
    Ft_Gpu_Hal_DeInit();
#ifdef MSVC_PLATFORM
	return 0;
#endif
}

void loop()
{
}


/* Nothing beyond this */













